@@ -4,6 +4,7 @@ use strict;
 use warnings;
 
 use Term::Screen;
+use Time::HiRes qw(sleep);
 
 use constant EVENT_SAW_FILE => "\x01";
 use constant EVENT_ENTER_FILE => "\x02";
@@ -36,7 +37,7 @@ sub new {
         files => [], 
         skip_files => {},
         last_line => 0,
-        skip_installed => 0,
+        skip_installed => $ENV{RR_SKIP_INC} // 0,
     }, $pkg;
 
     return $self;
@@ -51,6 +52,20 @@ sub _saw_file {
     $self->files->[$file_id] = $path;
 }
 
+sub _show_current_file {
+    my $self = shift;
+    
+    $screen->clrscr();
+    $screen->at(0, 0);
+
+    if ($self->current_file) {
+        $screen->bold->puts($self->current_file_path)->normal();
+    }
+    else {
+        $screen->bold->puts("Can't find file\n")->normal();                
+    }
+}
+
 sub _enter_file {
     my $self = shift;
     my ($buff);
@@ -60,8 +75,6 @@ sub _enter_file {
     close $self->current_file if $self->current_file;
     $self->{current_file_path} = $self->files->[$file_id];
 
-    $screen->clrscr();
-    $screen->at(0, 0);
 
     if (-e $self->current_file_path) {
         open my $file, "<", $self->current_file_path or die $!;
@@ -70,18 +83,56 @@ sub _enter_file {
         $self->{all_lines} = \@lines;
         $self->{num_lines} = @lines;
 
-        $screen->bold->puts($self->current_file_path)->normal();
     }
     else {
         $self->{current_file} = undef;
-        $screen->bold->puts("Can't find file\n")->normal();        
     }
     
+    $self->_show_current_file();
+    
     1;
 }
 
-my $site_libs = join "|", grep { /^\// } @INC;
-my $site_qr = qr{$site_libs};
+{
+    my $site_libs = join "|", grep { /^\// } @INC;
+    my $site_qr = qr{$site_libs};
+
+    sub _show_current_line {
+        my $self = shift;
+    
+        return unless $self->current_file;
+        return if $self->skip_files->{$self->current_file_path};
+        return if $self->skip_installed && $self->current_file_path =~ $site_qr;
+
+        my $line_no = $self->last_line;
+        
+        my $screen_cols = $screen->cols;
+        my $screen_rows = int(($screen->rows - 4) / 2);
+    
+        my $from = $line_no > $screen_rows ? $line_no - $screen_rows : 0;
+        my $to = $line_no + $screen_rows < $self->num_lines - 1 ? $line_no + $screen_rows : $self->num_lines - 1;
+    
+        # Adjust to fill screen
+        $to += ($screen->rows - 4 - ($to - $from)) if ($to - $from) < $screen->rows - 4;
+    
+        $screen->at(2, 0);
+        $screen->clreos();
+        my $p = length $self->num_lines;
+    
+        my $i = 0;
+        for my $l ($from..$to) {    
+            last if $l > $self->num_lines;
+            last unless defined $self->all_lines->[$l];
+            $screen->at(2 + $i++, 0);
+            $screen->reverse if $l == $line_no;
+            my $src = sprintf("% ${p}d: %s", $l, substr($self->all_lines->[$l], 0, $screen_cols - ($p + 2)));
+            $screen->puts($src);
+            $screen->normal if $l == $line_no;
+        }    
+
+        $self->_process_key();
+    }
+}
 
 sub _enter_line {
     my $self = shift;
@@ -92,30 +143,8 @@ sub _enter_line {
     $line_no--;
 
     $self->{last_line} = $line_no;
-    return unless $self->current_file;
-    return if $self->skip_files->{$self->current_file_path};
-    return if $self->skip_installed && $self->current_file_path =~ $site_qr;
     
-    my $screen_cols = $screen->cols;
-    my $screen_rows = int(($screen->rows - 4) / 2);
-    
-    my $from = $line_no > $screen_rows ? $line_no - $screen_rows : 0;
-    my $to = $line_no + $screen_rows < $self->num_lines - 1 ? $line_no + $screen_rows : $self->num_lines - 1;
-    
-    $screen->at(2, 0);
-    $screen->clreos();
-    my $p = length $self->num_lines;
-    
-    my $i = 0;
-    for my $l ($from..$to) {    
-        $screen->at(2 + $i++, 0);
-        $screen->reverse if $l == $line_no;
-        my $src = sprintf("% ${p}d: %s", $l, substr($self->all_lines->[$l], 0, $screen_cols - ($p + 2)));
-        $screen->puts($src);
-        $screen->normal if $l == $line_no;
-    }
-
-    $self->_process_key();
+    $self->_show_current_line();
 }
 
 sub _throw_exception {
@@ -131,12 +160,22 @@ my %KEY_HANDLER = (
         my $self = shift; 
         $self->skip_files->{$self->current_file_path} = 1; 
     },
-    a => sub { shift->{skip_installed} |= 1 },
+    a => sub { 
+        $_[0]->{skip_installed} ^= 1;
+        $screen->at(1, 0)->puts("Skip installed is: " . ((qw(OFF ON)[$_[0]->skip_installed])));
+        $screen->getch();
+    },
+    h => \&_show_help,
 );
 
 sub _process_key {
     my $self = shift;
     
+    if ($ENV{RR_AUTORUN}) {
+        sleep $ENV{RR_AUTORUN};
+        return;
+    }
+    
     my $k = lc $screen->getch();
     $screen->at(1, 0);
 
@@ -146,16 +185,38 @@ sub _process_key {
     1;
 }
 
+sub _show_help {
+    my $self = shift;
+    
+    $screen->clrscr();
+    $screen->at(0, 0)->puts("Help for 'rr-viewer'");
+    $screen->at(2, 0)->puts("'a' - Toggle skip files in \@INC at start");
+    $screen->at(3, 0)->puts("'s' - Skip the current file");
+    $screen->at(5, 0)->puts("'h' - Show this help");
+    $screen->at(6, 0)->puts("'q' - Quit prematurely");
+    $screen->at(8, 0)->puts("... press the ANY key to continue ...");
+    $screen->getch();
+    
+    $self->_show_current_file();
+    $self->_show_current_line();
+    
+}
 sub playback {
     my $self = shift;
 
     while (defined(my $event = $self->io->getc)) {
         $EVENT{$event}->($self);
     }
+    
+    $self->done;
 }
 
 sub done {
-    $screen->clrscr();
+    $screen->at(0, 0)->clreol();
+    $screen->at(0, 0)->puts("Playback completed, press the ANY key to quit...");
+  
+    $screen->getch();
+    
     exit 0;
 }
 
